package	jp.co.sds_corp.volcano;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.List;

import javax.imageio.ImageIO;
import javax.media.Buffer;
import javax.media.Format;
import javax.media.format.VideoFormat;
import javax.media.protocol.ContentDescriptor;
import javax.media.protocol.PullBufferStream;

public class ImageSourceStream implements PullBufferStream
{

	List<Image> images;
	int width, height;
	VideoFormat format;

	int nextImage = 0;	// index of the next image to be read.
	boolean ended = false;

	public ImageSourceStream(int width, int height, int frameRate, List<Image> images) {
		this.width = width;
		this.height = height;
		this.images = images;

		format = new VideoFormat(VideoFormat.JPEG,
				new Dimension(width, height),
				Format.NOT_SPECIFIED,
				Format.byteArray,
				(float )frameRate );
	}

	/**
	 * We should never need to block assuming data are read from files.
	 */
	public boolean willReadBlock() {
		return false;
	}

	/**
	 * This is called from the Processor to read a frame worth
	 * of video data.
	 */
 	public void read(Buffer buf) throws IOException {
		if (nextImage >= images.size()) {
System.err.println("Done reading all images.");
			buf.setEOM(true);
			buf.setOffset(0);
			buf.setLength(0);
			ended = true;
			return;
		}

		BufferedImage image = (BufferedImage)images.get(nextImage);
		nextImage++;

System.err.println("  - reading image(" + nextImage + ")" );

		byte data[] = null;

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		ImageIO.write( image, "jpeg", bos );
		data = bos.toByteArray();
		buf.setData(data);

		buf.setOffset(0);
		buf.setLength(data.length);
		buf.setFormat(format);
		buf.setFlags(buf.getFlags() | Buffer.FLAG_KEY_FRAME);
	}

	/**
	 * Return the format of each video frame.  That will be JPEG.
	 */
	public Format getFormat() {
		return format;
	}

	public ContentDescriptor getContentDescriptor() {
		return new ContentDescriptor(ContentDescriptor.RAW);
	}

	public long getContentLength() {
		return 0;
	}

	public boolean endOfStream() {
		return ended;
	}

	public Object[] getControls() {
		return new Object[0];
	}

	public Object getControl(String type) {
		return null;
	}
}

